==> garlicsim_py3-0.2.2.zip: garlicsim_py3-0.2.2/garlicsim_py3/synchronous_crunching/list_simulate.py <==
# Copyright 2009 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the list_simulate function. See its documentation for more
information.
'''

import copy
import warnings

from garlicsim_py3.general_misc import cute_iter_tools

import garlicsim_py3
import garlicsim_py3.misc
from . import history_browser as history_browser_module # Avoiding name clash

__all__ = ["list_simulate"]

def list_simulate(simpack, state, iterations, *args, **kwargs):
    '''
    Simulate from the given state for the given number of iterations.

    A simpack must be passed as the first parameter. Any extraneous parameters
    will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''
    simpack_grokker = garlicsim_py3.misc.SimpackGrokker(simpack)
    step_profile = garlicsim_py3.misc.StepProfile(*args, **kwargs)
    
    if not hasattr(state, 'clock'):
        state = copy.deepcopy(state)        
        state.clock = 0
                      
    if simpack_grokker.history_dependent:
        return __history_list_simulate(simpack_grokker, state, iterations,
                                       step_profile)
    else: # It's a non-history-dependent simpack
        return __non_history_list_simulate(simpack_grokker, state, iterations,
                                           step_profile)

    
def __history_list_simulate(simpack_grokker, state, iterations,
                             step_profile=None):
    '''
    For history-dependent simulations only:
    
    Simulate from the given state for the given number of iterations.

    A simpack must be passed as the first parameter. Any extraneous parameters
    will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''
    
    if step_profile is None: step_profile = garlicsim_py3.misc.StepProfile()
    
    tree = garlicsim_py3.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    history_browser = history_browser_module.HistoryBrowser(path)
    
    iterator = simpack_grokker.step_generator(history_browser, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root
    for current_state in finite_iterator:
        current_node = tree.add_state(current_state, parent=current_node)
    
    return [node.state for node in path]


def __non_history_list_simulate(simpack_grokker, state, iterations,
                                step_profile=None):
    '''
    For non-history-dependent simulations only:
    
    Simulate from the given state for the given number of iterations.

    A simpack must be passed as the first parameter. Any extraneous parameters
    will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''

    if step_profile is None: step_profile = garlicsim_py3.misc.StepProfile()
    
    tree = garlicsim_py3.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    
    iterator = simpack_grokker.step_generator(state, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root

    for current_state in finite_iterator:
        current_node = tree.add_state(current_state, parent=current_node)
    
    return [node.state for node in path]

==> garlicsim_py3-0.4.zip: garlicsim_py3-0.4/garlicsim_py3/synchronous_crunching/list_simulate.py <==
# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the list_simulate function.

See its documentation for more info.
'''

import copy
import warnings

from garlicsim_py3.general_misc import cute_iter_tools

import garlicsim_py3
import garlicsim_py3.misc
from . import history_browser as history_browser_module # Avoiding name clash

__all__ = ["list_simulate"]

def list_simulate(simpack, state, iterations, *args, **kwargs):
    '''
    Simulate from the given state for the given number of iterations.

    A simpack must be passed as the first parameter. Any extraneous parameters
    will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''
    simpack_grokker = garlicsim_py3.misc.SimpackGrokker(simpack)
    step_profile = garlicsim_py3.misc.StepProfile(*args, **kwargs)
    
    if not hasattr(state, 'clock'):
        state = copy.deepcopy(state)        
        state.clock = 0
                      
    if simpack_grokker.history_dependent:
        return __history_list_simulate(simpack_grokker, state, iterations,
                                       step_profile)
    else: # It's a non-history-dependent simpack
        return __non_history_list_simulate(simpack_grokker, state, iterations,
                                           step_profile)

    
def __history_list_simulate(simpack_grokker, state, iterations,
                             step_profile=None):
    '''
    Simulate from the given state for the given number of iterations.

    (Internal function for history-dependent simulations only.)    
    
    A simpack must be passed as the first parameter. Any extraneous parameters
    will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''
    
    if step_profile is None: step_profile = garlicsim_py3.misc.StepProfile()
    
    tree = garlicsim_py3.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    history_browser = history_browser_module.HistoryBrowser(path)
    
    iterator = simpack_grokker.step_generator(history_browser, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root
    for current_state in finite_iterator:
        current_node = tree.add_state(current_state, parent=current_node)
    
    return [node.state for node in path]


def __non_history_list_simulate(simpack_grokker, state, iterations,
                                step_profile=None):
    '''
    Simulate from the given state for the given number of iterations.
    
    (Internal function for non-history-dependent simulations only.)

    A simpack must be passed as the first parameter. Any extraneous parameters
    will be passed to the step function.
    
    Returns a list that spans all the states, from the initial one given to
    the final one.
    '''

    if step_profile is None: step_profile = garlicsim_py3.misc.StepProfile()
    
    tree = garlicsim_py3.data_structures.Tree()
    root = tree.add_state(state, parent=None)
    path = root.make_containing_path()
    
    iterator = simpack_grokker.step_generator(state, step_profile)
    finite_iterator = cute_iter_tools.shorten(iterator, iterations)
    
    current_node = root
    for current_state in finite_iterator:
        current_node = tree.add_state(current_state, parent=current_node)
    
    return [node.state for node in path]
